SystemOrganization addCategory: #'SqueakNxt-EtoysCompiler'!

!LegoNxtMorph methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 1/11/2011 08:08'!
playerTranslator
	^ NxtPlayerTranslator on: self! !

!NxtPluggableDeviceMorph methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 1/11/2011 08:23'!
playerTranslator
	"Looks for the brick in which it is connected and returns a NxtPlayerTranslator on it"
	| ports brick |
	ports := #(#port1 #port2 #port3 #port4 #portA #portB #portC ).
	brick := LegoNxtMorph allInstances
		detect: [:morph | ports anySatisfy: [:port | (morph nxt perform: port) = device port]]
		ifNone: [self error: 'The device is not plugged to any brick'].
	^ brick playerTranslator! !

!PhraseTileMorph methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 1/8/2011 13:49'!
color: aColor
self color: aColor of: self! !

!PhraseTileMorph methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 1/8/2011 13:56'!
color: aColor of: aMorph
| subm |
	subm := aMorph submorphs.
	subm do: [:each | "(each isKindOf: TileMorph) ifTrue: [each color: aColor] ifFalse: [self color: aColor of: each]" 
(each isKindOf: StringMorph orOf: ImageMorph) ifFalse: [each color: aColor.
							self color: aColor of: each]]! !

RBProgramNodeVisitor subclass: #NxtPlayerTranslator
	instanceVariableNames: 'brick motors sensors getterSelectors setterSelectors scriptSelectors variables methods currentMorph programName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-EtoysCompiler'!

!NxtPlayerTranslator commentStamp: 'Richo 1/11/2011 10:23' prior: 0!
A NxtPlayerTranslator is responsible for transforming a bunch of Etoys scripts (which may be compiled in a set of different Player subclasses) into a subclass of NxtProgram which can be translated to nxc.

This usually requires:
* compiling all methods and variables into one single class (it adds a prefix representing the receiver to avoid name clashes)
* renaming the message selectors to match the ones supported by nxc
* changing all receivers to self.

Apart from that, it also takes care of representing etoys multithreading using nxc tasks and flag variables to decide when the task should stop.

See #translateMainScript:of: as a starting point on how to use this class.

Instance Variables
	brick:		<LegoNxtMorph>
	motors:		<Array>
	sensors:		<Array>
	getterSelectors:		<Dictionary>
	setterSelectors:		<Dictionary>
	scriptSelectors:		<Dictionary>
	variables:		<Dictionary>
	methods:		<Set>
	currentMorph:		<Morph>
	programName:		<Symbol>


brick
	- An instance of LegoNxtMorph which contains the scripts we are going to translate.

motors
	- An array containing each motor connected to the nxt brick. Empty slots means nothing is connected in that port.

sensors
	- An array containing each sensor connected to the nxt brick. Empty slots means nothing is connected in that port.

getterSelectors
	- A dictionary containing all the slots getters. Each key is one of the morphs composing the robot, so we can know which slot corresponds to which morph.

setterSelectors
	- A dictionary containing all the slots setters. Each key is one of the morphs composing the robot, so we can know which slot corresponds to which morph.

scriptSelectors
	- A dictionary containing all the script selectors. Each key is one of the morphs composing the robot, so we can know which script corresponds to which morph.

variables
	- A dictionary containing all the instance variables (task flags included). Each key is one of the morphs composing the robot, so we can know which slot corresponds to which morph. Also, this dictionary stores the variable type apart from its name.
	
methods
	- A set of strings containing the methods' source code to be compiled.

currentMorph
	- A pointer to one of the morphs composing the robot. Since all methods *must* be sent to one of these morphs (external references are not allowed), it defines which morph is currently represented by the 'self' variable. Very useful, but you shouldn't modify it directly. Instead use #withCurrentMorph:do:.

programName
	- The name of the temporary class we are using to compile the code.!

!NxtPlayerTranslator class methodsFor: 'utilities' stamp: 'Richo 1/11/2011 08:59'!
avoidNameClash
"
Utility method. Compiles an empty method in SqueakNxtPlayer for each nxc primitive rule selector. This way we prevent the user from creating a script with an invalid name.
Execute this if the list of rules changes."
"
self avoidNameClash.
"
NxcPrimitiveRule allRulesSelectors, #(main setUp) 
	select: [:each | (each isKeyword and: [each numArgs = 1]) or: [each isUnary]]
	thenDo: [:selector | | method |
	method := String streamContents: [:stream |
		selector isUnary
			ifTrue: [stream nextPutAll: selector]
			ifFalse: [selector keywords doWithIndex: [:each :index | stream nextPutAll: each; space; nextPutAll: 'obj', index asString; space]].
		stream cr; nextPutAll: '"This method''s name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"'
		].
	"Warning, we don't want to lose an already existing method"
	(SqueakNxtPlayer canUnderstand: selector)
		ifFalse: [SqueakNxtPlayer compile: method classified: #'*squeaknxt-etoyscompiler-nxc commands']
].! !

!NxtPlayerTranslator class methodsFor: 'instance creation' stamp: 'Richo 1/10/2011 16:34'!
on: aLegoNxtMorph
^ (self basicNew setBrick: aLegoNxtMorph) initialize! !

!NxtPlayerTranslator methodsFor: 'visiting' stamp: 'Richo 1/12/2011 14:52'!
acceptMessageNode: aMessageNode 
	| selector |
	selector := aMessageNode selector.
	"Check for invalid selector or invalid receiver"
	self assert: [self isValidSelector: selector];		
		assert: [self isValidReceiver: aMessageNode receiver].
	"Is it a rule?"
	(self rules includesKey: selector)
		ifTrue: [(self rules at: selector)
				value: aMessageNode].
	"Is it an etoys message?"
	(self etoysMessages includesKey: selector)
		ifTrue: [(self etoysMessages at: selector)
				value: aMessageNode].
	"Is it a getter?"
	(self allGetterSelectors includes: selector)
		ifTrue: [| newNode | 
			self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: [newNode := RBVariableNode
								named: (self slotForGetter: selector)].
			aMessageNode
				become: (newNode parent: aMessageNode parent).
			^ self visitNode: aMessageNode].
	"Is it a setter?"
	(self allSetterSelectors includes: selector)
		ifTrue: [| newNode | 
			self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: [newNode := RBAssignmentNode
								variable: (RBVariableNode
										named: (self slotForSetter: selector))
								value: aMessageNode arguments first].
			aMessageNode
				become: (newNode parent: aMessageNode parent).
			^ self visitNode: aMessageNode].
	"Is it a script call?"
	(self allScriptSelectors includes: selector)
		ifTrue: [self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: ["Check for recursive call"
					self assert: [self isNotRecursiveCall: aMessageNode].
					aMessageNode
						selector: (self asLegalName: aMessageNode selector);
						
						receiver: (RBVariableNode named: 'self')]].
	"We still need to visit the receiver and the arguments"
	^ super acceptMessageNode: aMessageNode! !

!NxtPlayerTranslator methodsFor: 'visiting' stamp: 'Richo 1/12/2011 00:25'!
acceptMethodNode: aMethodNode 
	"It goes over the method's arguments adding the #argTypes: pragma if necessary and modifies the selector to have a legal name"
	aMethodNode allArgumentVariables size = 1 
		ifTrue: [| varType | 
			varType := self asNxcType: (self currentPlayerClass scripts at: aMethodNode selector) argumentVariables first variableType.
			aMethodNode pragmas: (RBParser parseMethod: 'm <argTypes:''' , varType , '''>') pragmas].
	aMethodNode
		selector: (self asLegalName: aMethodNode selector).
	^ super acceptMethodNode: aMethodNode! !

!NxtPlayerTranslator methodsFor: 'visiting' stamp: 'Richo 1/12/2011 13:28'!
acceptSequenceNode: aSequenceNode
	"If one of the statements has an invalid tile it will ignore it entirely. Only assertion failures will be ignored, other errors will be signaled"
	aSequenceNode statements do: [:each | [self visitNode: each] on: AssertionFailure do: [self becomeNoOp: each]]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 17:36'!
allGetterSelectors
	^ getterSelectors inject: #() into: [:subTotal :next | subTotal, next]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/11/2011 08:32'!
allMorphs
	"Returns a collection with all the morphs composing the robot (it could contain nils)"
	^ {brick}, motors, sensors! !

!NxtPlayerTranslator methodsFor: 'iterating' stamp: 'Richo 1/11/2011 08:32'!
allMorphsDo: aBlock
	"Iterates over all morphs composing the robot taking care of posible nil occurrences"
	self allMorphs do: [:each |
		each notNil ifTrue: [aBlock value: each]
	]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 17:36'!
allScriptSelectors
	^ scriptSelectors inject: #() into: [:subTotal :next | subTotal, next]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 17:36'!
allSetterSelectors
	^ setterSelectors inject: #() into: [:subTotal :next | subTotal, next]! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 1/10/2011 18:29'!
asLegalName: aString
"All variables, method names and task names must begin with a prefix representing the receiver."
^ currentMorph assuredPlayer uniqueNameForReference asLegalSelector, aString capitalized! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 1/11/2011 08:47'!
asLegalTaskFlag: aSymbol 
	"Returns the name of the flag variable used to stop this task"
	^ self asLegalName: (self asTaskFlag: aSymbol)! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 1/11/2011 08:45'!
asLegalTaskName: aSymbol
	"Returns the name of this task"
	^ self asLegalName: (self asTaskName: aSymbol)! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 1/11/2011 08:49'!
asNxcType: aSymbol
	""
	^ aSymbol caseOf: {
		[#Number] -> [#long].
		[#Boolean] -> [#bool].
		[#String] -> [#string]
		}! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 1/10/2011 18:52'!
asTaskFlag: aSymbol
	"Returns the name of the flag variable used to stop this task"
	^ (self asTaskName: aSymbol), 'Stepping'! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 1/10/2011 18:50'!
asTaskName: aSymbol
	"Returns the name of this task"
	^ aSymbol, 'Task'! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 1/12/2011 13:24'!
becomeNoOp: aParseNode
	"Transforms the argument in an empty statement:
	
	self nxc: ''.
	"
	| noOp |
	noOp := RBMessageNode receiver: (RBVariableNode named: 'self') selector: #nxc: arguments: {self literal: ''}.
	aParseNode become: (noOp parent: aParseNode parent)! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 1/11/2011 08:30'!
collectGetterSelectors
	"Collects all selectors corresponding to a slot getter in a dictionary whose keys are the morphs owning the slot"
	getterSelectors := Dictionary new.
	self allMorphsDo: [:morph | | selectors |
		selectors := getterSelectors at: morph ifAbsent: [getterSelectors at: morph put: OrderedCollection new].
		selectors addAll: (morph assuredPlayer slotInfo keys collect: [:selector | Utilities getterSelectorFor: selector])].
! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 1/11/2011 08:10'!
collectMotorsAndSensors
	"Finds out which motor and sensor morphs are connected to the nxt brick.
	It's not the fastest way (nor the cleanest) but it works..."
	| nxt |
	nxt := brick nxt.
	motors := Array new: 3.
	sensors := Array new: 4.
	NxtPluggableDeviceMorph allSubInstancesDo: [:m | 
		m device port caseOf: {
			[nxt portA] -> [motors at: 1 put: m].
			[nxt portB] -> [motors at: 2 put: m].
			[nxt portC] -> [motors at: 3 put: m].
			[nxt port1] -> [sensors at: 1 put: m].
			[nxt port2] -> [sensors at: 2 put: m].
			[nxt port3] -> [sensors at: 3 put: m].
			[nxt port4] -> [sensors at: 4 put: m].
		} otherwise: []
	]! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 1/11/2011 08:10'!
collectRelevantInformation
	self collectMotorsAndSensors; 
		 collectSlots;
		 collectGetterSelectors;
		 collectSetterSelectors;
		 collectScriptSelectors! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 1/11/2011 08:31'!
collectScriptSelectors
	"Collects all selectors corresponding to a script in a dictionary whose keys are the morphs owning the methods"
	scriptSelectors := Dictionary new.
	self
		allMorphsDo: [:morph | 
			| selectors | 
			selectors := scriptSelectors
						at: morph
						ifAbsent: [scriptSelectors at: morph put: OrderedCollection new].
			selectors
				addAll: (morph assuredPlayer class selectors
						reject: [:selector | ((getterSelectors at: morph)
									includes: selector)
								or: [(setterSelectors at: morph)
										includes: selector]])] ! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 1/11/2011 08:30'!
collectSetterSelectors
	"Collects all selectors corresponding to a slot setter in a dictionary whose keys are the morphs owning the slot"
	setterSelectors := Dictionary new.
	self allMorphsDo: [:morph | | selectors |
		selectors := setterSelectors at: morph ifAbsent: [setterSelectors at: morph put: OrderedCollection new].
		selectors addAll: (morph assuredPlayer slotInfo keys collect: [:selector | Utilities setterSelectorFor: selector])].
! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 1/11/2011 08:10'!
collectSlots
	"Adds all the slots and they respective types to the variables dictionary."
	variables := Dictionary new.
	self allMorphsDo: [:morph | | slots |
		slots := variables at: morph ifAbsent: [variables at: morph put: Dictionary new].
		morph assuredPlayer slotInfo keysAndValuesDo: [:slotName :slotInfo | slots at: slotName put: slotInfo type]]! !

!NxtPlayerTranslator methodsFor: 'translating' stamp: 'Richo 1/10/2011 18:51'!
compileMainMethod: scriptName
	"Compiles the main method, which is the first method the nxt executes. It sends #setUp to configure the sensors, then it starts the script task"
methods add: 'main 
	<task>
	self setUp.
	self nxc: ''StartTask(', (self asLegalTaskName: scriptName) ,');'''.
! !

!NxtPlayerTranslator methodsFor: 'compiling' stamp: 'Richo 1/11/2011 08:51'!
compileNxtProgramSubclass
	| instVarNames instVarTypes |
	instVarNames := instVarTypes := ''.
	self
		variablesDo: [:varName :varType | 
			instVarNames := instVarNames, (self asLegalName: varName) , ' '.
			instVarTypes := instVarTypes, (self asNxcType: varType) , ' ']. 
	self createNxtProgramSubclassInstVarNames: instVarNames instVarTypes: instVarTypes.
	methods
		do: [:each | self programClass compileSilently: each classified: #autogenerated]! !

!NxtPlayerTranslator methodsFor: 'translating' stamp: 'Richo 1/11/2011 08:53'!
compileSetUpMethod
"Compiles the #setUp method, which gets executed once at the beginning of the program and its responsible for configuring the sensors and initialize all the inst vars. It also waits for 500 milliseconds (just to be sure all the sensors are correctly configured)"
| methodSource |
methodSource := WriteStream on: ''.
methodSource nextPutAll: 'setUp'; cr.
sensors doWithIndex: [:m :index |
	m isNil ifFalse: [ | zeroBasedIndex |
		zeroBasedIndex := (index - 1) asString.
		methodSource nextPutAll: '	self setSensor'.
		m device type caseOf: {
			[SensorType lowspeed9V] -> [methodSource nextPutAll: 'LowSpeed: ', zeroBasedIndex].
			[SensorType lightActive] -> [methodSource nextPutAll: 'Light: ', zeroBasedIndex, ' active: true'].
			[SensorType lightInactive] -> [methodSource nextPutAll: 'Light: ', zeroBasedIndex, ' active: false'].
			[SensorType soundDba] -> [methodSource nextPutAll: 'Sound: ', zeroBasedIndex, ' scaling: false'].
			[SensorType soundDb] -> [methodSource nextPutAll: 'Sound: ', zeroBasedIndex, ' scaling: true'].
			[SensorType switch] -> [methodSource nextPutAll: 'Touch: ', zeroBasedIndex]
		} otherwise: [].
		methodSource nextPut: $.; cr.
	]
].
self variablesDo: [:varName :varType | 
	methodSource 
		nextPutAll: (self asLegalName: varName), ' := ' , (self currentPlayer instVarNamed: varName) printString , '.';
		cr].
methodSource nextPutAll: 'self wait: 500.'.
methods add: methodSource contents! !

!NxtPlayerTranslator methodsFor: 'translating' stamp: 'Richo 1/12/2011 12:41'!
compileTaskNamed: aSymbol 
	"Each task is defined as a loop whose condition is the task's flag variable. So, if we want to stop the task we just need to set the flag to false, and the task will stop at the next tick. This is a workaround, a correct implementation would call StopTask() but this function is only supported by the enhanced firmware and I *really* need to be able to stop tasks from Etoys, so this solution (even though far from perfect) is good enough. This code also takes into account the frequency and tickingRate parameters of the script. To do that, it encloses the script call in a #timesRepeat: message and it puts a #wait: afterwards "
	| rule |
	self assert: [(scriptSelectors at: currentMorph) includes: aSymbol].
	(variables at: currentMorph) at: (self asTaskFlag: aSymbol) put: #Boolean.
	rule := '@taskName 
	<task>
	@taskFlag := true. 
	[@taskFlag] 
		whileTrue: [@frequency timesRepeat: [self @task].
			self wait: @delay]'.
	rule := rule copyReplaceAll: '@taskName' with: (self asLegalTaskName: aSymbol).
	rule := rule copyReplaceAll: '@taskFlag' with: (self asLegalTaskFlag: aSymbol).
	rule := rule copyReplaceAll: '@task' with: (self asLegalName: aSymbol).
	rule := rule copyReplaceAll: '@frequency' with: (self frequencyOfScript: aSymbol).
	rule := rule copyReplaceAll: '@delay' with: (self delayOfScript: aSymbol).
	methods add: rule! !

!NxtPlayerTranslator methodsFor: 'compiling' stamp: 'Richo 1/10/2011 18:53'!
createNxtProgramSubclassInstVarNames: aString instVarTypes: anotherString 
	SystemChangeNotifier uniqueInstance doSilently: [
		NxtProgram subclass: self programName
			instanceVariableNames: aString
			instanceVariableTypes: anotherString
			classVariableNames: ''
			poolDictionaries: ''
			category: 'NxtPrograms']! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 17:22'!
currentPlayer
	^ currentMorph assuredPlayer! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 17:22'!
currentPlayerClass
	^ self currentPlayer class! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 18:15'!
delayOfScript: scriptName
 	"To be in accordance with the script tick rate, we put a delay at the end of each task. How long the delay would be?"
	^ (1000 / (currentMorph assuredPlayer scriptInstantiationForSelector: scriptName) tickingRate) rounded asString! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/12/2011 13:15'!
etoysMessages
"I don't know why but recent versions of Etoys use this weird messages instead of the operators."
^ {
#eToysLT: -> [:node | node selector: #<].
#eToysLE: -> [:node | node selector: #<=].
#eToysEQ: -> [:node | node selector: #=].
#eToysNE: -> [:node | node selector: #~=].
#eToysGT: -> [:node | node selector: #>].
#eToysGE: -> [:node | node selector: #>=].
} as: Dictionary! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 18:15'!
frequencyOfScript: scriptName
	"To be in accordance with the script frequency property we enclose the script call in a #timesRepeat: message. How many times should we repeat the script?"
	^ (currentMorph assuredPlayer scriptInstantiationForSelector: scriptName) frequency rounded asString! !

!NxtPlayerTranslator methodsFor: 'testing' stamp: 'Richo 1/12/2011 14:50'!
isNotRecursiveCall: aMessageNode
	^ (self asLegalName: aMessageNode selector) ~= aMessageNode methodNode selector! !

!NxtPlayerTranslator methodsFor: 'testing' stamp: 'Richo 1/12/2011 13:07'!
isValidReceiver: aParseNode
	"If the parse node is a variable it *must* be one of the valid morphs. I'm not sure about this, actually"
	aParseNode isVariable ifTrue: [^(self morphFor: aParseNode name) notNil].
	^ true! !

!NxtPlayerTranslator methodsFor: 'testing' stamp: 'Richo 1/12/2011 13:05'!
isValidSelector: selector 
	"Checks if the selector is one of the following:
	* A nxc primitive rule.
	* A player translator rule
	* An etoys message.
	* A slot accessor.
	* A script selector."
	^ NxcPrimitiveRule allRulesSelectors , self rules keys , self etoysMessages keys , self allGetterSelectors , self allSetterSelectors , self allScriptSelectors includes: selector! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 17:33'!
literal: aLiteralObject
	^ RBLiteralValueNode value: aLiteralObject! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/12/2011 12:56'!
morphFor: receiverName
	"Returns the morph represented by the argument or nil if it's not a valid morph"
	^ receiverName = 'self'
				ifTrue: [currentMorph]
				ifFalse: [self allMorphs
						detect: [:each | each notNil and: [each assuredPlayer uniqueNameForReference = receiverName]] ifNone: [nil]]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/11/2011 09:18'!
nxcIndexOf: aVariableNode in: aCollection
	"Basically, it tries to determine if the variable node is representing an object in the collection. If it is, it returns its index as a literal node (zero based for nxc compatibility)"
	| index |
	aVariableNode name = 'self' 
		ifTrue: [index := (aCollection indexOf: currentMorph)]
		ifFalse: [aCollection doWithIndex: [:morph :i| 
(morph notNil and: [morph assuredPlayer uniqueNameForReference = aVariableNode name]) ifTrue: [index := i. thisContext return: nil]]].
	^ index isNil ifTrue: [self error: 'Index not found'] ifFalse: [self literal: index - 1]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 18:54'!
programClass
	"Returns the temporary class we are using to compile the code"
	^ Smalltalk at: self programName ifAbsent: []! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 18:53'!
programName
	^ programName ifNil: [self programName: #NxtEtoysProgram]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 18:54'!
programName: aSymbol
	"It tries to set programName to aSymbol, but if the class already exists it appends as many R as necessary. It's just a temporary class and it will be removed right away"
	^ (Smalltalk includesKey: aSymbol)
		ifTrue: [self programName: (aSymbol , #R) asSymbol]
		ifFalse: [programName := aSymbol]! !

!NxtPlayerTranslator methodsFor: 'compiling' stamp: 'Richo 1/10/2011 18:56'!
removeNxtProgramSubclass
	"After translation is done we silently remove the temporary class"
	SystemChangeNotifier uniqueInstance doSilently: [
		self programClass ifNotNilDo: [:class | class removeFromSystem]]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/12/2011 00:41'!
rules
"These are the rules to modify the parse tree of the SqueakNxtPlayer scripts. Most of the time, I just need to rename the selector, change the receiver to self and set the correct list of arguments. Other times (such as #getAudible or #getActive) the rule is a little more complicated"
| selfVar |
selfVar := RBVariableNode named: 'self'.
^ {
#getIsPressed -> [:node | 
	node arguments: {self nxcIndexOf: node receiver in: sensors};
		 selector: #sensorBoolean:;
		receiver: selfVar].
#playTone: -> [:node | 
	node receiver: selfVar;
		  arguments: node arguments, {self literal: 200};
		  selector: #playTone:duration:].
#setPower: -> [:node | 
	node arguments: {self nxcIndexOf: node receiver in: motors} , node arguments;
		selector: #onFwd:speed:;
		receiver: selfVar].
#getAudible -> [:node | | copy | "(self sensorType: n) = 8"
	copy := node copy.
	node receiver: copy; arguments: {self literal: SensorType soundDba}; selector: #=.
	copy arguments: {self nxcIndexOf: copy receiver in: sensors}; selector: #sensorType:; receiver: selfVar].
#getActive -> [:node | | copy | "(self sensorType: n) = 5"
	copy := node copy.
	node receiver: copy; arguments: {self literal: SensorType lightActive}; selector: #=.
	copy arguments: {self nxcIndexOf: copy receiver in: sensors}; selector: #sensorType:; receiver: selfVar].
#getDistanceCm -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: sensors};
		 selector: #sensorUS:;
		receiver: selfVar].
#getValue -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: sensors};
			selector: #sensorValue:;
			receiver: selfVar].
#float -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: motors};
		selector: #float:;
		receiver: selfVar].
#brake -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: motors};
		selector: #off:;
		receiver: selfVar].
#playSoundFile: -> [:node | node selector: #playFile:; receiver: selfVar].
#getPower -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: motors};
		selector: #motorPower:;
		receiver: selfVar].
#setActive: -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: sensors}, node arguments;
		 selector: #setSensorLight:active:;
		receiver: selfVar].
#setAudible: -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: sensors}, {RBMessageNode receiver: node arguments first selector: #not};
		 selector: #setSensorSound:scaling:;
		receiver: selfVar].
#startScript: -> [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [	
	self compileTaskNamed: node arguments first value.
	node arguments: {self literal: 'StartTask(', (self asLegalTaskName: node arguments first value) ,');'};
		 selector: #nxc:;
		 receiver: selfVar]].
#stopScript: -> [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [	
	self compileTaskNamed: node arguments first value.
	node arguments: {self literal: (self asLegalTaskFlag: node arguments first value), ' = false;'};
		 selector: #nxc:;
		 receiver: selfVar]].
#pauseScript: -> [:node | "Do the same as #stopScript:"
	(self rules at: #stopScript:) value: node].
#doScript: -> [:node | | args | args := node arguments. "This is the same as calling the script directly"
	node arguments: #(); selector: args first value.
	self visitNode: node "We need to visit the node again to fix the selector and the receiver"]
} as: Dictionary! !

!NxtPlayerTranslator methodsFor: 'iterating' stamp: 'Richo 1/11/2011 08:55'!
scriptSelectorsDo: aBlock 
	"Iterates over the scriptSelectors dictionary taking care of setting the current morph"
	scriptSelectors
		keysAndValuesDo: [:morph :selectors | self
				withCurrentMorph: morph
				do: [selectors do: aBlock]] ! !

!NxtPlayerTranslator methodsFor: 'private' stamp: 'Richo 1/12/2011 12:36'!
setBrick: aMorph
	"By default we set the brick as the currentMorph"
	brick := currentMorph := aMorph! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/11/2011 08:34'!
slotForGetter: getterSelector
	"Returns the slot accessed by these selector"
	| slotName |
	slotName := currentMorph assuredPlayer slotInfo keys
				detect: [:slot | (Utilities getterSelectorFor: slot)
						= getterSelector].
	^ self asLegalName: slotName! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/11/2011 08:35'!
slotForSetter: setterSelector
	"Returns the slot accessed by these selector"
	| slotName |
	slotName := currentMorph assuredPlayer slotInfo keys
				detect: [:slot | (Utilities setterSelectorFor: slot)
						= setterSelector].
	^ self asLegalName: slotName! !

!NxtPlayerTranslator methodsFor: 'translating' stamp: 'Richo 1/12/2011 12:36'!
translateMainScript: scriptName
	^ self translateMainScript: scriptName of: currentMorph! !

!NxtPlayerTranslator methodsFor: 'translating' stamp: 'Richo 1/12/2011 15:10'!
translateMainScript: scriptName of: scriptOwner
	"It translates the script to nxc. It creates a temporary NxtProgram subclass to hold the code which will be translated to nxc. After translation is done, it removes the temporary class.
	Steps:
	* Collect relevant information (motors and sensors composing the robot, slots, accessors and scripts).
	* Compile #setUp method.
	* Compile #main method.
	* Compile the script task.
	* Compile all scripts.
	* Compile NxtProgram subclass.
	* Translate program to nxc
	* Remove temporary class.
	* Return nxc program"
	self collectRelevantInformation.
	self
		assert: [self allMorphs includes: scriptOwner]
		description: 'The morph is not connected to the Nxt brick';
		assert: [scriptOwner assuredPlayer class scripts includesKey: scriptName]
		description: 'The script doesn''t belong to the morph'.
	methods := Set new. 
	self
		withCurrentMorph: scriptOwner
		do: [self compileSetUpMethod; compileMainMethod: scriptName; compileTaskNamed: scriptName].
	self
		scriptSelectorsDo: [:each | 
			| node | 
			self visitNode: (node := self currentPlayerClass parseTreeFor: each).
			methods add: node formattedCode].
	[self compileNxtProgramSubclass.
	^ self programClass asNxcProgram: scriptName]
		ensure: [self removeNxtProgramSubclass]! !

!NxtPlayerTranslator methodsFor: 'iterating' stamp: 'Richo 1/11/2011 08:54'!
variablesDo: aBlock 
	"Iterates over the variables dictionary taking care of setting the current morph"
	variables
		keysAndValuesDo: [:morph :vars | self
				withCurrentMorph: morph
				do: [vars keysAndValuesDo: aBlock]] ! !

!NxtPlayerTranslator methodsFor: 'translating' stamp: 'Richo 1/11/2011 08:44'!
withCurrentMorph: aMorph do: aBlock
	"It executes the block using the morph argument as a our current morph. It also remembers the old morph, so it's safe to nest this expression.
	The current morph is used whenever we need a legal name for a variable/method name. It defines which morph is currently represented by the 'self' variable"
	| old |
	old := currentMorph.
	currentMorph := aMorph.
	aBlock value.
	currentMorph := old! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
and: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
arcCos
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
arcSin
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
arcTan
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/11/2011 09:00'!
atRandom
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
ceiling
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
cos
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
degreeArcTan
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
degreeCos
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
degreeSin
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
degreeTan
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
doWhileTrue: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
exp
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
float: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
floor
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
ifFalse: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
ifTrue: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
ln
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
log
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
main
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
motorPower: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
motorSpeed: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
negated
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 1/10/2011 14:35'!
newScriptorAround: aPhrase
	Transcript show: 'newScriptorAround:'; cr.
	^ super newScriptorAround: aPhrase! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
not
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
off: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
or: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
playFile: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
raisedTo: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/11/2011 09:00'!
random
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
repeat
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 1/10/2011 15:32'!
scriptEditorFor: aSelector
	Transcript show: 'scriptEditorFor:'; cr.
	^ super scriptEditorFor: aSelector! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 1/10/2011 15:32'!
scriptEditorForNoCng: aSelector
	Transcript show: 'scriptEditorForNoCng:'; cr.
	^ super scriptEditorForNoCng: aSelector.! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
sensorBoolean: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
sensorNormalized: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
sensorRaw: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
sensorScaled: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
sensorType: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
sensorUS: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
sensorValue: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
setSensorLight: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
setSensorLowSpeed: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
setSensorSound: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
setSensorTouch: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
setUp
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
sin
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
sqrt
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
tan
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
timesRepeat: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
truncated
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
wait: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
whileTrue
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 1/5/2011 17:10'!
whileTrue: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !
